/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>



#define VER         "0.1.1"
#define KEYPROXY    "Software\\RiverFront\\WebDrive"
#define KEYPATH     "Software\\RiverFront\\WebDrive\\Connections"
#define BUFFSZ      8192
#define B64DECX     bb = *bf; \
                    for(; \
                    bb && \
                    ((bb == ' ') || \
                    (bb == '\t') || \
                    (bb == '\r') || \
                    (bb == '\n')); \
                    bf++);



void readreg(HKEY key, u_char *buff, u_char *value);
void webdrive_pwd(unsigned char *pwd);
unsigned char *Base64Decode(unsigned char *buf, unsigned char *ret);



int main(void) {
    HKEY            key,
                    keypwd;
    unsigned char   keybuff[sizeof(KEYPATH) + BUFFSZ],
                    value[BUFFSZ + 1],
                    *ptr;
    int             bufflen,
                    k;


    setbuf(stdout, NULL);

    fputs("\n"
        "Webdrive passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(RegOpenKeyEx(HKEY_CURRENT_USER, KEYPATH, 0, KEY_READ, &key)) {
        fputs("\n"
            "Error: registry key has not been found\n"
            "\n"
            "    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    strcpy(keybuff, KEYPATH);
    ptr = keybuff + sizeof(KEYPATH) - 1;
    *ptr++ = '\\';
    k = 0;
    while(1) {
        bufflen = BUFFSZ;
        if(RegEnumKeyEx(
            key,
            k++,
            ptr,
            (void *)&bufflen,
            NULL,
            NULL,
            NULL,
            NULL)) break;

        printf("\n%s\n", ptr);

        RegOpenKeyEx(HKEY_CURRENT_USER, keybuff, 0, KEY_READ, &keypwd);
        readreg(keypwd, value, "URL");
        printf("  Address:    %s\n", value);

        readreg(keypwd, value, "UserName");
        printf("  Username:   %s\n", value);

        readreg(keypwd, value, "PassWord");
        webdrive_pwd(value);
        printf("  PassWord:   %s\n", value);

        RegCloseKey(keypwd);
    }
    RegCloseKey(key);

    fputs("\n### Proxies:\n", stdout);
    RegOpenKeyEx(HKEY_CURRENT_USER, KEYPROXY, 0, KEY_READ, &keypwd);
    readreg(keypwd, value, "FirewallServer");
    printf("  Proxy-FTP Address:     %s:", value);
    readreg(keypwd, value, "FirewallPort");
    printf("%d\n", *(int *)value);
    readreg(keypwd, value, "FirewallUsername");
    printf("  Proxy-FTP Username:    %s\n", value);
    readreg(keypwd, value, "FirewallPassword");
    webdrive_pwd(value);
    printf("  Proxy-FTP Password:    %s\n", value);

    fputc('\n', stdout);

    readreg(keypwd, value, "HttpProxyAddress");
    printf("  Proxy-HTTP Address:    %s:", value);
    readreg(keypwd, value, "HttpProxyPort");
    printf("%d\n", *(int *)value);
    readreg(keypwd, value, "HttpProxyUsername");
    printf("  Proxy-HTTP Username:   %s\n", value);
    readreg(keypwd, value, "HttpProxyPassword");
    webdrive_pwd(value);
    printf("  Proxy-HTTP Password:   %s\n", value);

    RegCloseKey(keypwd);

    fputs("\n\n   Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void readreg(HKEY key, u_char *buff, u_char *value) {
    int     valuelen = BUFFSZ;

    if(RegQueryValueEx(
        key,
        value,
        NULL,
        NULL,
        buff,
        (void *)&valuelen)) *value = 0;
}



void webdrive_pwd(unsigned char *pwd) {
    if(*pwd != 0x02) return;
    Base64Decode(pwd + 1, pwd);
}



unsigned char *Base64Decode(unsigned char *buf, unsigned char *ret) {
    int             c,
                    d,
                    i,
                    *Base;
    unsigned char   bb,
                    *bf,
                    *rf;

    if(!*buf) {
        *ret = 0;
        return(ret);
    }

    Base = (int *)malloc(sizeof(int) << 8);
    if(!Base) {
        strcpy(ret, buf);
        return(ret);
    }
    for(i = 0; i < 26; i++) {
        Base['A' + i] = i;
        Base['a' + i] = i + 26;
    }
    for(i = 0; i < 10; i++) {
        Base['0' + i] = i + 52;
    }
    Base['+'] = 62;
    Base['/'] = 63;

    bf = buf;
    rf = ret;
    B64DECX;
    while(1) {
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        d = Base[bb];
        *rf++ = (c << 2) | (d >> 4);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];
        *rf++ = ((d & 0xF) << 4) | (c >> 2);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        *rf++ = ((c & 0x3) << 6) | Base[bb];
    }
    *rf = 0;
    free(Base);
    return(rf);
}


